package com.gic.operation.web.controller.gray;

import com.gic.api.base.commons.Constant;
import com.gic.api.base.commons.Page;
import com.gic.commons.exception.BaseRuntimeException;
import com.gic.commons.exception.ExceptionManager;
import com.gic.commons.httpclient.HttpClientUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.dsmongo.api.dto.Criteria;
import com.gic.dsmongo.api.dto.MongoDelBatchDTO;
import com.gic.dsmongo.api.dto.MongoQueryDTO;
import com.gic.dsmongo.api.dto.MongoSaveDTO;
import com.gic.gray.config.dto.GrayConfig;
import com.gic.gray.config.dto.GrayModuleConfig;
import com.gic.gray.config.dto.JavaModuleDeployInfo;
import com.gic.gray.config.util.GrayConfigUtil;
import com.gic.gray.config.util.MongoOperaUtil;
import com.gic.operation.web.qo.GrayConfigQo;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.stream.Collectors;


/**
 * 灰度管理controller
 *
 * @author jiaotianqi
 * @date 2021/7/26 10:39
 */
@RestController
@RequestMapping("/gray")
public class GrayManagerController {

    @RequestMapping("/add-gray-config")
    public RestResponse addGrayConfig(@Validated @RequestBody GrayConfigQo grayConfigQo) {


        MongoQueryDTO mongoQueryDTO = new MongoQueryDTO();
        mongoQueryDTO.setDataBaseAndCollectionName("gic-deploy-git-4", "gray_module_config", "id");
        mongoQueryDTO.setCriteria(Criteria.where("_id").eq(GrayConfigUtil.getClusterName() + ":" + grayConfigQo.getModuleName()));
        Map result = MongoOperaUtil.get(mongoQueryDTO).getResult();
        if (result != null) {
            ExceptionManager.throwException("模块已经存在", BaseRuntimeException.class);
        }

        GrayModuleConfig grayModuleConfig = new GrayModuleConfig();
        grayModuleConfig.setId(GrayConfigUtil.getClusterName() + ":" + grayConfigQo.getModuleName());
        grayModuleConfig.setCluster_name(GrayConfigUtil.getClusterName());
        grayModuleConfig.setFront_end(grayConfigQo.getFrontEnd());
        grayModuleConfig.setGray_enable(grayConfigQo.getGrayEnable());
        grayModuleConfig.setGray_type(grayConfigQo.getGrayType());
        grayModuleConfig.setModule_name(grayConfigQo.getModuleName());

        MongoSaveDTO saveDto = new MongoSaveDTO();
        saveDto.setDataBaseAndCollectionName("gic-deploy-git-4", "gray_module_config", "id");
        saveDto.setData(grayModuleConfig);
        boolean save = MongoOperaUtil.save(saveDto);
        if (save) {
            GrayConfigUtil.refreshGrayConfig(Arrays.asList(grayConfigQo.getModuleName()));
        }
        return RestResponse.success(save);
    }


    @RequestMapping("/update")
    public RestResponse update(@Validated @RequestBody GrayConfigQo grayConfigQo) {

        GrayConfig newGrayConfig = new GrayConfig();
        newGrayConfig.setGrayEnable(grayConfigQo.getGrayEnable());
        newGrayConfig.setGrayType(grayConfigQo.getGrayType());
        if (CollectionUtils.isNotEmpty(grayConfigQo.getApplicationNameList())) {
            GrayConfigUtil.updateGrayConfig(newGrayConfig, grayConfigQo.getApplicationNameList());
        }
        return RestResponse.success();
    }


    @RequestMapping("/delete")
    public RestResponse delete(@RequestBody GrayConfigQo grayConfigQo) {
        MongoDelBatchDTO delDto=new MongoDelBatchDTO();
        MongoQueryDTO mongoQueryDTO = new MongoQueryDTO();
        mongoQueryDTO.setCriteria(Criteria.where("_id").eq(GrayConfigUtil.getClusterName()+":"+grayConfigQo.getModuleName()));
        delDto.setCriterias(mongoQueryDTO);
        delDto.setDataBaseAndCollectionName("gic-deploy-git-4", "gray_module_config", "id");
        boolean delete = MongoOperaUtil.delete(delDto);
        return RestResponse.success(delete);
    }

    @RequestMapping("/query-by-page")
    public RestResponse<Page<JavaModuleDeployInfo>> queryByPage(String moduleName, Boolean grayFlag, Integer currPage, Integer pageSize, Boolean frontEnd) {
        Page<JavaModuleDeployInfo> javaModuleDeployInfoPage = GrayConfigUtil.queryModuleConfig(currPage, pageSize, moduleName, grayFlag, frontEnd);
        return RestResponse.success(javaModuleDeployInfoPage);
    }


    @RequestMapping("/refresh-redis-gray-config")
    public RestResponse refreshRedisGrayConfig(HttpServletRequest request) {
        GrayConfigUtil.refreshGrayConfig(null, true, false);
        GrayConfigUtil.refreshGrayConfigChangeTimeByZk();
        String res = HttpClientUtil.get("https://" + request.getServerName() +"/gray-config-refresh");
        return RestResponse.success(res);
    }


    /**
     * 查询集群名称
     *
     * @return
     */
    @RequestMapping("/query-cluster-name")
    public RestResponse<List<String>> queryClusterName() {
        MongoQueryDTO queryDto = new MongoQueryDTO();
        queryDto.setShowField("_id");
        queryDto.setDataBaseAndCollectionName("gic-deploy-git-4", "cluster_config", "id");
        List<String> clusterNameList = MongoOperaUtil.query(queryDto).getResult().stream().map(d -> d.get("id").toString()).filter(d -> !d.endsWith("-gray")).collect(Collectors.toList());
        return RestResponse.success(clusterNameList);
    }


    /**
     * 查询模块名称
     *
     * @param type 1 灰度管理页面 2 路由管理页面
     * @return
     */
    @RequestMapping("/query-module-name")
    public RestResponse<List<String>> queryModuleName(@RequestParam(defaultValue = "1") Integer type) {

        MongoQueryDTO queryModuleConfig = new MongoQueryDTO();
        queryModuleConfig.setShowField("_id,is_dubbo_service");
        queryModuleConfig.setDataBaseAndCollectionName("gic-deploy-git-4", "java_module_config", "_id");
        List<String> moduleNameList = null;
        if (Objects.equals(type, Constant.NUMBER_1)) {
            MongoQueryDTO queryDto = new MongoQueryDTO();
            queryDto.setShowField("module_name");
            queryDto.setCriteria(Arrays.asList(Criteria.where("cluster_name").eq(GrayConfigUtil.getClusterName()),
                    Criteria.where("front_end").eq(false)));
            queryDto.setDataBaseAndCollectionName("gic-deploy-git-4", "gray_module_config", "_id");
            Set<String> moduleNameSet = MongoOperaUtil.query(queryDto).getResult().stream().map(d -> d.get("module_name").toString()).collect(Collectors.toSet());
            moduleNameList = MongoOperaUtil.query(queryModuleConfig).getResult().stream().map(d -> d.get("id").toString()).filter(d -> !moduleNameSet.contains(d)).collect(Collectors.toList());

        } else {
            MongoQueryDTO queryDto = new MongoQueryDTO();
            queryDto.setShowField("_id");
            queryDto.setCriteria(Criteria.where("front_end").eq(false));
            queryDto.setDataBaseAndCollectionName("gic-deploy-git-4", "nginx_module_config", "_id");
            Set<String> moduleNameSet = MongoOperaUtil.query(queryDto).getResult().stream().map(d -> d.get("id").toString()).collect(Collectors.toSet());
            moduleNameList = MongoOperaUtil.query(queryModuleConfig).getResult().stream().filter(
                    d -> !moduleNameSet.contains(d.get("id").toString()) && Optional.ofNullable(d.get("is_dubbo_service")).orElse("").toString().endsWith("false")).map(d -> d.get("id").toString()).collect(Collectors.toList());
        }
        return RestResponse.success(moduleNameList);
    }


}
