package com.gic.operation.web.controller.gray;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.exception.BaseRuntimeException;
import com.gic.commons.exception.ExceptionManager;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.dsmongo.api.dto.*;
import com.gic.dsmongo.api.utils.MongoUtil;
import com.gic.gray.config.dto.NginxLocationConfig;
import com.gic.gray.config.util.MongoOperaUtil;
import com.gic.operation.web.qo.RouteConfigQo;
import com.gic.operation.web.vo.RouteConfigVo;
import com.google.common.base.Splitter;
import jodd.util.StringUtil;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.stream.Collectors;


/**
 * 路由管理
 *
 * @author jiaotianqi
 * @date 2021/8/5 9:54
 */
@RestController
@RequestMapping("route")
public class RouteManagerController {


    @RequestMapping("/add-route-config")
    public RestResponse addRouteConfig(@RequestBody RouteConfigQo routeConfigQo) {

        Map nginxModuleConfig = getNginxModuleConfig(routeConfigQo.getModuleName());
        if(nginxModuleConfig!=null){
            ExceptionManager.throwException("模块已存在", BaseRuntimeException.class);
        }

        MongoSaveDTO saveDto = new MongoSaveDTO();
        saveDto.setDataBaseAndCollectionName("gic-deploy-git-4", "nginx_module_config", "id");

        Map<String, Object> data = new HashMap<>();
        data.put("id", routeConfigQo.getModuleName());
        data.put("location_regex", routeConfigQo.getRegxSymbol());
        data.put("location_route", routeConfigQo.getRouteRegx());
        data.put("module_path", routeConfigQo.getModulePath());
        data.put("front_end", routeConfigQo.getFrontEnd());
        if (routeConfigQo.getLocationConfig() != null && StringUtil.isNotBlank(routeConfigQo.getLocationConfig())) {
            data.put("location_config", Splitter.on(",").splitToList(routeConfigQo.getLocationConfig()));
        }
        saveDto.setData(data);
        boolean save = MongoOperaUtil.save(saveDto);
        return RestResponse.success(save);
    }

    @RequestMapping("/update-route-config")
    public RestResponse updateRouteConfig(@RequestBody RouteConfigQo routeConfigQo) {

        MongoUpdateBatchDTO updateDataDto = new MongoUpdateBatchDTO();
        updateDataDto.setDataBaseAndCollectionName("gic-deploy-git-4", "nginx_module_config", "id");

        Map<String, Object> data = new HashMap<>();
        data.put("id", routeConfigQo.getModuleName());
        data.put("location_regex", routeConfigQo.getRegxSymbol());
        data.put("location_route", routeConfigQo.getRouteRegx());
        data.put("module_path", routeConfigQo.getModulePath());
        data.put("front_end", routeConfigQo.getFrontEnd());

        updateDataDto.setJsonData(JSON.toJSONString(Arrays.asList(data), SerializerFeature.WriteMapNullValue));
        boolean updateBatch = MongoOperaUtil.updateBatch(updateDataDto);
        return RestResponse.success(updateBatch);
    }
    @RequestMapping("/delete")
    public RestResponse delete(@RequestBody RouteConfigQo routeConfigQo) {
        MongoDelBatchDTO delDto=new MongoDelBatchDTO();
        MongoQueryDTO mongoQueryDTO = new MongoQueryDTO();
        mongoQueryDTO.setCriteria(Criteria.where("_id").eq(routeConfigQo.getModuleName()));
        delDto.setCriterias(mongoQueryDTO);
        delDto.setDataBaseAndCollectionName("gic-deploy-git-4", "nginx_module_config", "id");
        boolean delete = MongoOperaUtil.delete(delDto);
        return RestResponse.success(delete);
    }

    @RequestMapping("/query-route-config")
    public RestResponse<Page<RouteConfigVo>> queryRouteConfig(@RequestBody RouteConfigQo routeConfigQo) {
        Page pageRes = new Page();

        MongoQueryDTO queryDto = new MongoQueryDTO();
        queryDto.setDataBaseAndCollectionName("gic-deploy-git-4", "nginx_module_config", "id");
        queryDto.setCurrentPage(routeConfigQo.getPageNum());
        queryDto.setPageSize(routeConfigQo.getPageSize());
        queryDto.setSort("_id=asc");
        List<Criteria> queryCriteriaList = new ArrayList<>();
        Optional.ofNullable(routeConfigQo.getModuleName()).ifPresent(v -> queryCriteriaList.add(Criteria.where("_id").regex(v)));
        Optional.ofNullable(routeConfigQo.getFrontEnd()).ifPresent(v -> queryCriteriaList.add(Criteria.where("front_end").eq(v)));
        queryDto.setCriteria(queryCriteriaList);

        ServiceResponse<Page<Map>> pageServiceResponse = MongoOperaUtil.queryPage(queryDto);

        if (pageServiceResponse.isSuccess()&&pageServiceResponse.getResult()!=null) {
            Page<Map> result = pageServiceResponse.getResult();
            pageRes.setCurrentPage(result.getCurrentPage());
            pageRes.setPageSize(result.getPageSize());
            pageRes.setTotalPage(result.getTotalPage());
            pageRes.setTotalCount(result.getTotalCount());
            pageRes.setResult(result.getResult().stream().map(d -> {
                RouteConfigVo routeConfigVo = new RouteConfigVo();
                routeConfigVo.setFrontEnd((Boolean) d.get("front_end"));
                routeConfigVo.setModuleName(d.get("id").toString());
                routeConfigVo.setRegxSymbol(d.get("location_regex").toString());
                routeConfigVo.setRouteRegx(d.get("location_route").toString());
                routeConfigVo.setModulePath(d.get("module_path").toString());
                return routeConfigVo;
            }).collect(Collectors.toList()));
        }
        return RestResponse.success(pageRes);
    }

    /**
     * 回显
     * @param moduleName
     * @return
     */
    @RequestMapping("/preview")
    public RestResponse preview(String moduleName) {

        NginxLocationConfig nginxLocationConfig = MongoUtil.toBean(getNginxModuleConfig(moduleName), NginxLocationConfig.class);

        StringBuilder nginxLocationConfigStr = new StringBuilder();

        nginxLocationConfigStr.append("location ").append(nginxLocationConfig.getLocation_regex()).append(" ").append(nginxLocationConfig.getLocation_route()).append("{ \r\n");

        if (nginxLocationConfig.getFront_end()) {
            //前端
            if (nginxLocationConfig.getLocation_config() != null) {
                for (String locationConfig : nginxLocationConfig.getLocation_config()) {
                    nginxLocationConfigStr.append("  ").append(locationConfig).append(";\r\n");
                }
            }else {
                nginxLocationConfigStr.append("  set_by_lua_file $target_upstream $grayRoute ").append(nginxLocationConfig.getId()).append(" ").append(nginxLocationConfig.getId()).append(" ").append(nginxLocationConfig.getId()).append("-gray 1;\r\n");
                nginxLocationConfigStr.append("  rewrite ^").append(nginxLocationConfig.getLocation_route()).append("/(.*)$ /$target_upstream/$1 break;\r\n");
                nginxLocationConfigStr.append("  try_files $uri $uri/ /$target_upstream/index.html;\r\n}\r\n\r\n\r\n");
            }

        } else {
            //后端
            List<String> locationConfigList = null;
            if (nginxLocationConfig.getLocation_config() != null) {
                locationConfigList = nginxLocationConfig.getLocation_config();
            } else {
                MongoQueryDTO otherConfigParam = new MongoQueryDTO();
                otherConfigParam.setCriteria(Criteria.where("_id").eq("nginx_location_default_config"));
                otherConfigParam.setDataBaseAndCollectionName("gic-deploy-git-4", "other_config", "id");
                locationConfigList = (List<String>) MongoOperaUtil.get(otherConfigParam).getResult().get("location_config");
            }
            for (String locationConfig : locationConfigList) {
                nginxLocationConfigStr.append("  ").append(locationConfig).append(";\r\n");
            }
            nginxLocationConfigStr.append("  set_by_lua_file $target_upstream $grayRoute ").append(nginxLocationConfig.getId()).append(" ").append(nginxLocationConfig.getId()).append(" ").append(nginxLocationConfig.getId()).append("-gray;\r\n")
                    .append("  rewrite ^").append(nginxLocationConfig.getLocation_route()).append("/(.*)$ ");
             if(StringUtil.isNotBlank(nginxLocationConfig.getModule_path())){
                 nginxLocationConfigStr.append("/").append(nginxLocationConfig.getModule_path());
             }
             nginxLocationConfigStr.append("/$1 break;\r\n")
                    .append("  proxy_pass http://$target_upstream;\r\n}");
        }
        return RestResponse.success(nginxLocationConfigStr.toString());
    }

    private Map getNginxModuleConfig(String moduleName) {
        MongoQueryDTO getDto = new MongoQueryDTO();
        getDto.setCriteria(Criteria.where("_id").eq(moduleName));
        getDto.setDataBaseAndCollectionName("gic-deploy-git-4", "nginx_module_config", "id");

        return MongoOperaUtil.get(getDto).getResult();
    }
}
